#!/etc/opt/openit/perl

BEGIN {
    push(@INC,"/opt/openit/lib" );
}

use strict;
use warnings;
use Getopt::Long;
use File::Copy qw(move);
use File::Basename;
use OpeniT::Config;
use OpeniT::DataTypes;


my $installationlog;
my $opt_debug;
my $opt_verbose;
my $opt_help;


unless ( GetOptions("help" => \ $opt_help,
         "debug"           => \ $opt_debug,
         "verbose"         => \ $opt_verbose
)) {
    &print_usage;
}

my $configdir = &get_configuration_dir();

# Update xml/conf files
if ($^O eq 'MSWin32' ){
	&_update_xml();
}
else
{
	&_update_daft_conf( "$configdir/daft.conf" ); 
	&_update_rawarchive_conf( "$configdir/rawarchiveregenerator.conf" );
}

sub _update_rawarchive_conf
{
	my ($filter) = @_;
	&create_backup($filter);
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read file '$filter'";
    my @lines = <FILE>;
	
	if ( grep {/autodesk/} @lines )
	{
		VERBOSE( "daft is already updated" );
		return;
    }
	
	
	my $ind = 0;
	
	$ind++ while( $lines[$ind] !~ /mapping/ );
	$ind++ while( $lines[$ind] !~ /wibukey/ );
	$ind++;
	splice @lines, $ind, 0, "		autodesk(String)=\"PollCollector.system.registry\"{}\n";
	_save_config( $filter, @lines );
}
sub _update_daft_conf
{

	my ($filter) = @_;
	&create_backup($filter);
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
	
	if ( grep {/autodesk/} @lines )
	{
		VERBOSE( "daft is already updated" );
		return;
    }
	
	
	my $ind = 0;
	
	$ind++ while( $lines[$ind] !~ /openit2.DirName/ );
	$ind++;
	splice @lines, $ind, 0, "		autodesk(DirName)=\"autodesk\"{}\n";
	
	$ind++ while( $lines[$ind] !~ /filtrate/ );
	$ind++ while( $lines[$ind] !~ /openit2.String/ );
	$ind++;
	splice @lines, $ind, 0, "		autodesk(String)=\"license-autodesk-inventory \$BEG \$END P1D\"{}\n";
	
	$ind++ while( $lines[$ind] !~ /datagroups/ );
	$ind++ while( $lines[$ind] !~ /openit2.String/ );
	$ind++;
	splice @lines, $ind, 0, "		autodesk(String)=\"PollParserAutodesk.system.autodesk\"{}\n";
	$ind++;
	
	my @insertpollcollector = ("		pollcollector(String)=\"PollCollector.license.codemeter_dongle,PollCollector.license.wibukey_dongle,PollCollector.license.vector_dongle,PollCollector.system.registry\"{}\n");
	
	# Pollcollector config already exist
	#just add value
	if ( grep {/.*PollCollector.license.codemeter_dongle.*/} @lines )
	{
		s/PollCollector.license.codemeter.*vector_dongle/PollCollector.license.codemeter_dongle,PollCollector.license.wibukey_dongle,PollCollector.license.vector_dongle,PollCollector.system.registry/g foreach(@lines);
	}
	else
	{
		splice @lines, $ind, 0, @insertpollcollector;
	}
	
	_save_config( $filter, @lines );
	
}

&_update_modules( "$configdir/modules" );
&_update_data_generation_config( "$configdir/data_generation_config" );
&_update_acc_types("$configdir/acc_types");
&_update_classvar("$configdir/classvar_mapping_config");
#&_update_confbuilder_conf( "$configdir/ConfBuilder-server.conf" );


exit 0;



sub _update_confbuilder_conf
{
	my ($filter) = @_;
	&create_backup($filter);
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;

	
	if ( grep {/autodesk/} @lines )
	{
		VERBOSE( "Confbuilder server conf is  is already updated" );
		return;
    }
	
	my $ind = $#lines;
	$ind++;
	
	my @insert = (
	"\n\"Components/filtrator/license-autodesk-inventory.conf\"	{ \"Components/filtrator/license-autodesk-inventory.xml\" }\n" );
	
	splice @lines, $ind, 0, @insert;
	
	_save_config( $filter, @lines );
}

sub _update_classvar
{
	my ($filter) = @_;
	&create_backup($filter);
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;

	
	if ( grep {/Autodesk/} @lines )
	{
		VERBOSE( "classvar_mapping_config is  is already updated" );
		return;
    }
	
	my $ind = $#lines;
	$ind-- while( $lines[$ind] !~ /\)/ );
	$ind-- while( $lines[$ind] !~ /\)\)/ );
	my $tmp = $lines[$ind] ;
	chomp $tmp;
	$lines[$ind] = "$tmp,";
	$ind++;
	
	splice @lines, $ind, 0, "\n";

	
	my @insert = (
	"\n  (Autodesk^software^inventory,\n",
	"	(),\n",
	"  ((Product,Product^id,),(Product, Feature),(Feature,Serial^number),(Feature,Release^number),(Feature,License^type), (Domain,Host), (Host,Operating^system) ))\n",
	"\n");

	
	splice @lines, $ind, 0, @insert;
	_save_config( $filter, @lines );
	
	
	
}

sub _update_xml()
{
	#update archiver.xml
	
	&_update_archiver_xml( "$configdir/Components/archiver.xml" ); 
	&_update_datagroup_generation( "$configdir/Components/datagroup-generation.xml" );
	
}

sub _update_data_generation_config
{
	my ($filter) = @_;
	&create_backup($filter);
	VERBOSE( "Updating $filter" );
	open(FILE,'>>', $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
	
	
	if ( grep {/Autodesk/} @lines )
	{
		VERBOSE( "data_generation_config is  is already updated" );
		return;
    }
	
	my $insert = 
"\n\nTYPE:inventory:Software inventory format:dynamic
118:Autodesk Software Inventory:license-autodesk-inventory,P1D:on\n";
	
	print FILE "$insert";
	close (FILE);
	
}


sub _update_acc_types
{
	my ($filter) = @_;
	&create_backup($filter);
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;

	
	if ( grep {/Autodesk/} @lines )
	{
		VERBOSE( "acc_types is already updated" );
		return;
    }
	
	
	my $ind = $#lines;
	# get last ')' 
	$ind-- while( $lines[$ind] !~ /\)/ );
	$ind--;
	$ind-- while( $lines[$ind] =~ /^\s*$/ );
	my $tmp = $lines[$ind] ;
	chomp $tmp;
	$lines[$ind] = "$tmp,";
	$ind++;
	
	splice @lines, $ind, 0, "\n";
	
	
	
	my @insert115 = 
	(
		" (Hostgroup^license^use^v3,\n",
			"  (\n",
			"   (Count,Record^Count,x,SUM),\n",
			"   (Elapsed^time,Accumulated^time^for^group^in^hours,Hour,SUM),\n",
			"   (Max^used,Maximum^concurrent^licenses^used^by^group,x,MAX),\n",
			"   (Avg^used,Average^concurrent^licenses^used^by^group,x,ABSAVG),\n",
			"   (Min^used,Minimum^concurrent^licenses^used^by^group,x,MIN),\n",
			"   (Duration, Duration^for^group^in^hours,Hour,SUM),\n",
			"   (License^cost,License^cost^given^in^seconds^*^cost^factor,Hour,WGT,\n",
			"    (Elapsed^time,((RANGE,olap_cpu_cost))))\n",
			"  ),\n",
			"  (Product,\n",
			"   Package,\n",
			"   Feature,\n",
			"   Feature^version,\n",
			"   Featureset,\n",
			"   Featureset^version,\n",
			"   Group^type,\n",
			"   Group,\n",
			"   License^type)\n",
			"  ), \n"
	);
	
	my @insert116 = 
	(
		" (Total^PPU^license^use^logfile,\n",
			"  (\n",
			"   (Count,Record^Count,x,SUM),\n",
			"   (Running^time,Running^time^in^hours,Hour,SUM),\n",
			"   (Avg^in^use,Average^number^of^licenses^in^use,x,ABSAVG),\n",
			"   (Max^in^use,Maximum^number^of^licenses^in^use,x,MAX),\n",
			"   (Min^in^use,Minimum^number^of^licenses^in^use,x,MIN),\n",
			"   (Elapsed^time,Elapsed^license^time^in^hours,Hour,SUM),\n",
			"   (Elastic^usage,Elastic^usage^time^in^hours,Hour,SUM),\n",
			"   (Distinct,Always^one^for^each^extracted^unique^classification,x,IDENT)\n",
			"  ),\n",
			"  (Customer^id,\n",
			"   Product,\n",
			"   Package,\n",
			"   License^type,\n",
			"   Feature,\n",
			"   Usage^type,\n",
			"   Max^available^licenses,\n",
			"   Used^licenses)\n",
			"  ), \n"
	);
	
	my @insert117 = 
	(
		" (Dongle^data^inventory,\n",
			"  (\n",
			"   (Count,Record^Count,x,SUM),\n",
			"   (Running^time,Accumulated^time^for^group^in^hours,Hour,SUM)\n",
			"  ),\n",
			"  (Dongle^type,\n",
			"   Serial^number,\n",
			"   Vendor,\n",
			"   Vendor^code,\n",
			"   Feature,\n",
			"   Feature^code,\n",
			"   Host^name)\n",
			"  ), \n"
	);

	my @insert118 = 
		(
			" (Autodesk^software^inventory,\n",
			"  (\n",
			"   (Count,Record^Count,x,SUM)\n",
			"  ),\n",
			"  (Product,\n",
			"   Product^id,\n",
			"   Feature,\n",
			"   Serial^number,\n",
			"   Release^number,\n",
			"   License^type,\n",
			"   Domain,\n",
			"   Host,\n",
			"   Operating^system)\n",
			"  ) \n"
	);
	
	
	my @data_types =  list_data_type_ids();
	my $no_of_dt  =  $#data_types;
	DEBUG ("Number of datatypes: $no_of_dt");
	
	$no_of_dt++;
	my @datatypeids = ( $no_of_dt .. 117 );
	my @holder =() ;
	
	foreach my $dt ( @datatypeids )
	{
		my @dummy =(
			"\n",
			"  (Placeholder^for^datatype^$dt,\n",
			"   ((open)),\n",
			"   (open)),\n",
			"\n" 			
		);
		if ( $dt eq 115 )
		{
			push (@holder , @insert115 );
		}	
		if ( $dt eq 116 )
		{
			push (@holder , @dummy );
		}	
		if ( $dt eq 117 )
		{
			push (@holder , @insert117 );	
		}
			
		
	}
	
	my @insert = ( "\n" );
	push (@insert, @holder );
	push (@insert, @insert118 );
	
	#find last line
	$ind = $#lines;
	$ind-- while( $lines[$ind] !~ /\)/ );
	$ind--;
	
	
	splice @lines, $ind, 0, @insert;
	_save_config( $filter, @lines );
	
}

sub _update_modules
{
	my ($filter) = @_;
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;

	
	if ( grep {/118/} @lines )
	{
		VERBOSE( "Modules is  is already updated" );
		return;
    }
	
	my $ind = 0;
	
	$ind++ while( $lines[$ind] !~ /System.Analyzer/ );
	$lines[$ind] = " (Open^iT^System^Analyzer, (0, 2, 3, 4, 5, 6, 27, 28, 31, 32, 33, 34, 35, 36, 48, 53, 54, 78, 80, 81, 82, 87, 118 ))\n";
		
	&_save_config( $filter, @lines );
	
}

sub _update_datagroup_generation
{
	
	my ($filter) = @_;
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
    close(FILE);
	
	if ( grep {/autodesk/} @lines )
	{
		VERBOSE( "datagroup-generation.xml filter is already updated" );
		return;
    }
	
	
	my $ind = 0;
	$ind++ while( $lines[$ind] !~ /openit2/ );
	$ind++ while( $lines[$ind] !~ /<\/Object>/ );
	$ind+=1;

	
	
	#insert Archive location
	my @insert = 
	(	"\n					<Object>\n",
		"						<Name>autodesk</Name>\n",
		"						<Description>Data types for raw autodesk inventory</Description>\n",
		"						<Value type=\"String\">PollParserAutodesk.system.autodesk</Value>\n",
		"					</Object>\n",
	);
	splice @lines, $ind, 0, @insert;
	
	my @insertpollcollector = 
	(
		"\n					<Object>\n",
		"						<Name>pollcollector</Name>\n",
		"						<Description>Data types for raw dongle inventory</Description>\n",
		"						<Value type=\"String\">PollCollector.license.codemeter_dongle,PollCollector.license.wibukey_dongle,PollCollector.license.vector_dongle,PollCollector.system.registry</Value>\n",
		"					</Object>\n"
	);
	
	# Pollcollector config already exist
	#just add value
	if ( grep {/.*PollCollector.license.codemeter_dongle.*/} @lines )
	{
		s/PollCollector.license.codemeter.*vector_dongle/PollCollector.license.codemeter_dongle,PollCollector.license.wibukey_dongle,PollCollector.license.vector_dongle,PollCollector.system.registry/g foreach(@lines);
	}
	else
	{
		splice @lines, $ind, 0, @insertpollcollector;
	}
	

	
	_save_config( $filter, @lines );
}

sub _update_archiver_xml
{
	
	my ($filter) = @_;
	VERBOSE( "Updating $filter" );

	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
    close(FILE);
	
	if ( grep {/autodesk/} @lines )
	{
		VERBOSE( "Archiver filter is already updated" );
		return;
    }
	
	my $ind = 0;
	
	$ind++ while( $lines[$ind] !~ /unsupported.handling/ );
	$ind++ while( $lines[$ind] !~ /\/Object/ );
	$ind++;
	
	
	

	my @insert =();
	
	
	$ind++ while( $lines[$ind] !~ /DirName.*openit2/ );
	$ind+=2;

	
	
	#insert Archive location
	@insert = 
	(	"					<Object>\n",
		"						<Name>autodesk</Name>\n",
		"						<Description>If type=autodesk, run the following filter processes:</Description>\n",
		"						<Value type=\"DirName\">autodesk</Value>\n",
		"					</Object>\n"
	);
	
	splice @lines, $ind, 0, @insert;
	
	#insert Filtrate command
	@insert = ();
	
	$ind++ while( $lines[$ind] !~ /openit2/ );
	$ind++ while( $lines[$ind] !~ /type.openit2/ );
	$ind+=3;
	
	@insert = 
	(	"					<Object>\n",
		"						<Name>autodesk</Name>\n",
		"						<Description>If type=autodesk, run the following filter processes:</Description>\n",
		"						<Value type=\"String\">license-autodesk-inventory \$BEG \$END P1D</Value>\n",
		"					</Object>\n"
	);
	
	splice @lines, $ind, 0, @insert;

	_save_config( $filter, @lines );
}

sub _save_config
{
	
	my ( $cfg, @lines ) = @_; 
	#&create_backup($cfg);
	open(OUT, ">$cfg") || die "Unable to overwrite filter file '$cfg'";
    print OUT @lines;
    close(OUT);
}

sub create_backup 
{
	my ($filter) = @_;
	DEBUG ( "Creating $filter backup " );
	my $new = "$filter.new";
    my $old = "$filter.old";

    open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
    close(FILE);
	
	open(OUT, ">$old") || die "Unable to create backup file '$old'";
    print OUT @lines;
    close(OUT);
}

sub get_configuration_dir()
{
	my $configdir;
	if ($^O eq 'MSWin32' )
	{
		my $cmd = `openit_freemould --script installation_info.oms`;
		chomp $cmd;
	
		$configdir =$cmd . "Configuration"
	}
	else
	{
		$configdir = `. /tmp/openit-cfg-shenv && echo \$CONFIG_DIR`;
	}
	chomp $configdir;
	DEBUG ("Got Confidir '$configdir'");
	return $configdir;
}


sub DEBUG 
{
	my ($log) = @_;
	$installationlog .= $log;
	if ( $opt_debug )
	{
		print ( "DEBUG: $log\n" );
	}
}

sub VERBOSE 
{
	my ($log) = @_;
	$installationlog .= $log;
	if ( $opt_verbose )
	{
		print ( "-- $log\n" );
	}
}

sub NOTICE 
{
	my ($log) = @_;
	$installationlog .= $log;
	print ( "NOTICE: $log\n" );
	
}

sub _grep_and_replace
{
	my ( $lines,  $grep_string, $new_value ) = @_;
	s/$grep_string/$new_value/g foreach(@$lines);
}


